@@ -2,7 +2,7 @@
 /*
  * This file is part of the libCEC(R) library.
  *
- * libCEC(R) is Copyright (C) 2011 Pulse-Eight Limited.  All rights reserved.
+ * libCEC(R) is Copyright (C) 2011-2012 Pulse-Eight Limited.  All rights reserved.
  * libCEC(R) is an original work, containing original code.
  *
  * libCEC(R) is a trademark of Pulse-Eight Limited.
@@ -34,7 +34,9 @@
 #ifndef CECEXPORTS_H_
 #define CECEXPORTS_H_
 
-#include <cectypes.h>
+#include "cectypes.h"
+
+#define LIBCEC_VERSION_CURRENT CEC_SERVER_VERSION_1_6_2
 
 namespace CEC
 {
@@ -48,7 +50,7 @@ namespace CEC
     /*!
      * @brief Open a connection to the CEC adapter.
      * @param strPort The path to the port.
-     * @param iTimeout Connection timeout in ms.
+     * @param iTimeoutMs Connection timeout in ms.
      * @return True when connected, false otherwise.
      */
     virtual bool Open(const char *strPort, uint32_t iTimeoutMs = 10000) = 0;
@@ -89,21 +91,25 @@ namespace CEC
     //@}
 
     /*!
+     * @deprecated Use libcec_configuration instead
      * @return Get the minimal version of libcec that this version of libcec can interface with.
      */
     virtual int8_t GetMinLibVersion(void) const = 0;
 
     /*!
+     * @deprecated Use libcec_configuration instead
      * @return Get the major version of libcec.
      */
     virtual int8_t GetLibVersionMajor(void) const = 0;
 
     /*!
+     * @deprecated Use libcec_configuration instead
      * @return Get the minor version of libcec.
      */
     virtual int8_t GetLibVersionMinor(void) const = 0;
 
     /*!
+     * @deprecated Use callback methods instead
      * @brief Get the next log message in the queue, if there is one.
      * @param message The next message.
      * @return True if a message was passed, false otherwise.
@@ -111,6 +117,7 @@ namespace CEC
     virtual bool GetNextLogMessage(cec_log_message *message) = 0;
 
     /*!
+     * @deprecated Use callback methods instead
      * @brief Get the next keypress in the queue, if there is one.
      * @param key The next keypress.
      * @return True if a key was passed, false otherwise.
@@ -118,8 +125,9 @@ namespace CEC
     virtual bool GetNextKeypress(cec_keypress *key) = 0;
 
     /*!
+     * @deprecated Use callback methods instead
      * @brief Get the next CEC command that was received by the adapter.
-     * @param action The next command.
+     * @param command The next command.
      * @return True when a command was passed, false otherwise.
      */
     virtual bool GetNextCommand(cec_command *command) = 0;
@@ -146,6 +154,7 @@ namespace CEC
     virtual bool SetPhysicalAddress(uint16_t iPhysicalAddress = CEC_DEFAULT_PHYSICAL_ADDRESS) = 0;
 
     /*!
+     * @deprecated Use libcec_configuration instead.
      * @brief Enable physical address detection (if the connected adapter supports this).
      * @return True when physical address detection was enabled, false otherwise.
      */
@@ -196,6 +205,7 @@ namespace CEC
     /*!
      * @brief Change the deck info, if this adapter is a playback device.
      * @param info The new deck info.
+     * @param bSendUpdate True to send the status over the CEC line.
      * @return True if set, false otherwise.
      */
     virtual bool SetDeckInfo(cec_deck_info info, bool bSendUpdate = true) = 0;
@@ -216,7 +226,7 @@ namespace CEC
 
     /*!
      * @brief Display a message on the device with the given logical address.
-     * @param iLogicalAddres The device to display the message on.
+     * @param iLogicalAddress The device to display the message on.
      * @param duration The duration of the message
      * @param strMessage The message to display.
      * @return True when the command was sent, false otherwise.
@@ -235,7 +245,7 @@ namespace CEC
      * @param iLogicalAddress The device to get the CEC version for.
      * @return The version or CEC_VERSION_UNKNOWN when the version couldn't be fetched.
      */
-    virtual cec_version GetDeviceCecVersion(cec_logical_address iAddress) = 0;
+    virtual cec_version GetDeviceCecVersion(cec_logical_address iLogicalAddress) = 0;
 
     /*!
      * @brief Get the menu language of the device with the given logical address
@@ -243,35 +253,35 @@ namespace CEC
      * @param language The requested menu language.
      * @return True when fetched succesfully, false otherwise.
      */
-    virtual bool GetDeviceMenuLanguage(cec_logical_address iAddress, cec_menu_language *language) = 0;
+    virtual bool GetDeviceMenuLanguage(cec_logical_address iLogicalAddress, cec_menu_language *language) = 0;
 
     /*!
      * @brief Get the vendor ID of the device with the given logical address.
      * @param iLogicalAddress The device to get the vendor id for.
      * @return The vendor ID or 0 if it wasn't found.
      */
-    virtual uint64_t GetDeviceVendorId(cec_logical_address iAddress) = 0;
+    virtual uint64_t GetDeviceVendorId(cec_logical_address iLogicalAddress) = 0;
 
     /*!
      * @brief Get the power status of the device with the given logical address.
      * @param iLogicalAddress The device to get the power status for.
      * @return The power status or CEC_POWER_STATUS_UNKNOWN if it wasn't found.
      */
-    virtual cec_power_status GetDevicePowerStatus(cec_logical_address iAddress) = 0;
+    virtual cec_power_status GetDevicePowerStatus(cec_logical_address iLogicalAddress) = 0;
 
     /*!
      * @brief Get the physical address of the device with the given logical address.
      * @param iLogicalAddress The device to get the vendor id for.
      * @return The physical address or 0 if it wasn't found.
      */
-    virtual uint16_t GetDevicePhysicalAddress(cec_logical_address iAddress) = 0;
+    virtual uint16_t GetDevicePhysicalAddress(cec_logical_address iLogicalAddress) = 0;
 
     /*!
      * @brief Sends a POLL message to a device.
-     * @param iAddress The device to send the message to.
+     * @param iLogicalAddress The device to send the message to.
      * @return True if the POLL was acked, false otherwise.
      */
-    virtual bool PollDevice(cec_logical_address iAddress) = 0;
+    virtual bool PollDevice(cec_logical_address iLogicalAddress) = 0;
 
     /*!
      * @return The devices that are active on the bus and not handled by libcec.
@@ -280,10 +290,10 @@ namespace CEC
 
     /*!
      * @brief Check whether a device is active on the bus.
-     * @param iAddress The address to check.
+     * @param iLogicalAddress The address to check.
      * @return True when active, false otherwise.
      */
-    virtual bool IsActiveDevice(cec_logical_address iAddress) = 0;
+    virtual bool IsActiveDevice(cec_logical_address iLogicalAddress) = 0;
 
     /*!
      * @brief Check whether a device of the given type is active on the bus.
@@ -332,10 +342,10 @@ namespace CEC
 
     /*!
      * @brief Get the OSD name of a device on the CEC bus.
-     * @param iAddress The device to get the OSD name for.
+     * @param iLogicalAddress The device to get the OSD name for.
      * @return The OSD name.
      */
-    virtual cec_osd_name GetDeviceOSDName(cec_logical_address iAddress) = 0;
+    virtual cec_osd_name GetDeviceOSDName(cec_logical_address iLogicalAddress) = 0;
 
     /*!
      * @brief Get the logical address of the device that is currently the active source on the CEC bus.
@@ -345,10 +355,29 @@ namespace CEC
 
     /*!
      * @brief Check whether a device is currently the active source on the CEC bus.
-     * @param iAddress The address to check.
+     * @param iLogicalAddress The address to check.
      * @return True when it is the active source, false otherwise.
      */
-    virtual bool IsActiveSource(cec_logical_address iAddress) = 0;
+    virtual bool IsActiveSource(cec_logical_address iLogicalAddress) = 0;
+
+    /*!
+     * @brief Sets the stream path to the device on the given logical address.
+     * @param iLogicalAddress The address to activate.
+     * @return True when the command was sent, false otherwise.
+     */
+    virtual bool SetStreamPath(cec_logical_address iLogicalAddress) = 0;
+
+    /*!
+     * @brief Sets the stream path to the device on the given logical address.
+     * @param iPhysicalAddress The address to activate.
+     * @return True when the command was sent, false otherwise.
+     */
+    virtual bool SetStreamPath(uint16_t iPhysicalAddress) = 0;
+
+    /*!
+     * @return The list of addresses that libCEC is controlling
+     */
+    virtual cec_logical_addresses GetLogicalAddresses(void) = 0;
 
     virtual const char *ToString(const cec_menu_state state) = 0;
     virtual const char *ToString(const cec_version version) = 0;
@@ -360,6 +389,53 @@ namespace CEC
     virtual const char *ToString(const cec_system_audio_status mode) = 0;
     virtual const char *ToString(const cec_audio_status status) = 0;
     virtual const char *ToString(const cec_vendor_id vendor) = 0;
+    virtual const char *ToString(const cec_client_version version) = 0;
+    virtual const char *ToString(const cec_server_version version) = 0;
+
+    /*!
+     * @brief Get libCEC's current configuration.
+     * @param configuration The configuration.
+     * @return True when the configuration was updated, false otherwise.
+     */
+    virtual bool GetCurrentConfiguration(libcec_configuration *configuration) = 0;
+
+    /*!
+     * @brief Change libCEC's configuration.
+     * @param configuration The new configuration.
+     * @return True when the configuration was changed successfully, false otherwise.
+     */
+    virtual bool SetConfiguration(const libcec_configuration *configuration) = 0;
+
+    /*!
+     * @return True when this device can persist the user configuration, false otherwise.
+     */
+    virtual bool CanPersistConfiguration(void) = 0;
+
+    /*!
+     * @brief Persist the given configuration in adapter (if supported)
+     * @brief The configuration to store.
+     * @return True when the configuration was persisted, false otherwise.
+     */
+    virtual bool PersistConfiguration(libcec_configuration *configuration) = 0;
+
+    /*!
+     * @brief Tell libCEC to poll for active devices on the bus.
+     */
+    virtual void RescanActiveDevices(void) = 0;
+
+    /*!
+     * @return true when libCEC is the active source on the bus, false otherwise.
+     */
+    virtual bool IsLibCECActiveSource(void) = 0;
+
+    /*!
+     * @brief Get information about the given device
+     * @param strPort The port to which the device is connected
+     * @param config The device configuration
+     * @param iTimeoutMs The timeout in milliseconds
+     * @return True when the device was found, false otherwise
+     */
+    virtual bool GetDeviceInformation(const char *strPort, libcec_configuration *config, uint32_t iTimeoutMs = 10000) = 0;
   };
 };
 
@@ -367,19 +443,23 @@ namespace CEC
  * @brief Load the CEC adapter library.
  * @param strDeviceName How to present this device to other devices.
  * @param deviceTypes The device types to use on the CEC bus.
+ * @param iPhysicalAddress The physical address to assume on the bus. If set to 0, libCEC will try to autodetect the address, with the data provided via SetHDMIPort()
  * @return An instance of ICECAdapter or NULL on error.
  */
-extern "C" DECLSPEC void * CECInit(const char *strDeviceName, CEC::cec_device_type_list devicesTypes);
+extern "C" DECLSPEC void * CECInit(const char *strDeviceName, CEC::cec_device_type_list deviceTypes, uint16_t iPhysicalAddress = 0);
 
 /*!
- * @deprecated Please use CECInit() instead
  * @brief Load the CEC adapter library.
- * @param strDeviceName How to present this device to other devices.
- * @param iLogicalAddress The logical of this device. PLAYBACKDEVICE1 by default.
- * @param iPhysicalAddress The physical address of this device. 0x1000 by default.
+ * @param configuration The configuration to pass to libCEC
  * @return An instance of ICECAdapter or NULL on error.
  */
-extern "C" DECLSPEC void * CECCreate(const char *strDeviceName, CEC::cec_logical_address iLogicalAddress = CEC::CECDEVICE_PLAYBACKDEVICE1, uint16_t iPhysicalAddress = CEC_DEFAULT_PHYSICAL_ADDRESS);
+extern "C" DECLSPEC void * CECInitialise(CEC::libcec_configuration *configuration);
+
+/*!
+ * @brief Try to connect to the adapter and send the "start bootloader" command, without initialising libCEC and going through all checks
+ * @return True when the command was send, false otherwise.
+ */
+extern "C" DECLSPEC bool CECStartBootloader(void);
 
 /*!
  * @brief Unload the CEC adapter library.
