@@ -354,7 +354,8 @@ namespace CecSharp
 		Version1_5_2  = 0x1502,
 		Version1_5_3  = 0x1503,
 		Version1_6_0  = 0x1600,
-		Version1_6_1  = 0x1601
+		Version1_6_1  = 0x1601,
+    Version1_6_2  = 0x1602
 	};
 
 	public enum class CecServerVersion
@@ -365,7 +366,8 @@ namespace CecSharp
 		Version1_5_2  = 0x1502,
 		Version1_5_3  = 0x1503,
 		Version1_6_0  = 0x1600,
-		Version1_6_1  = 0x1601
+		Version1_6_1  = 0x1601,
+    Version1_6_2  = 0x1602
 	};
 
 	public ref class CecAdapter
@@ -580,10 +582,13 @@ namespace CecSharp
 
 			PowerOffScreensaver = CEC_DEFAULT_SETTING_POWER_OFF_SCREENSAVER == 1;
 			PowerOffOnStandby   = CEC_DEFAULT_SETTING_POWER_OFF_ON_STANDBY == 1;
-      SendInactiveSource  = CEC_DEFAULT_SETTING_SEND_INACTIVE_SOURCE == 1;
-      LogicalAddresses    = gcnew CecLogicalAddresses();
-      FirmwareVersion     = 1;
-      PowerOffDevicesOnStandby = CEC_DEFAULT_SETTING_POWER_OFF_DEVICES_STANDBY == 1;
+
+			SendInactiveSource  = CEC_DEFAULT_SETTING_SEND_INACTIVE_SOURCE == 1;
+			LogicalAddresses    = gcnew CecLogicalAddresses();
+			FirmwareVersion     = 1;
+			PowerOffDevicesOnStandby = CEC_DEFAULT_SETTING_POWER_OFF_DEVICES_STANDBY == 1;
+			ShutdownOnStandby   = CEC_DEFAULT_SETTING_SHUTDOWN_ON_STANDBY == 1;
+			DeviceLanguage      = "";
 		}
 
 		void SetCallbacks(CecCallbackMethods ^callbacks)
@@ -624,22 +629,26 @@ namespace CecSharp
 			PowerOffScreensaver = config.bPowerOffScreensaver == 1;
 			PowerOffOnStandby = config.bPowerOffOnStandby == 1;
 
-      if (ServerVersion >= CecServerVersion::Version1_5_1)
-        SendInactiveSource = config.bSendInactiveSource == 1;
+			if (ServerVersion >= CecServerVersion::Version1_5_1)
+				SendInactiveSource = config.bSendInactiveSource == 1;
 
-      if (ServerVersion >= CecServerVersion::Version1_5_3)
-      {
-        LogicalAddresses->Clear();
-			  for (uint8_t iPtr = 0; iPtr <= 16; iPtr++)
-				  if (config.logicalAddresses[iPtr])
-  					LogicalAddresses->Set((CecLogicalAddress)iPtr);
-      }
+			if (ServerVersion >= CecServerVersion::Version1_5_3)
+			{
+				LogicalAddresses->Clear();
+				for (uint8_t iPtr = 0; iPtr <= 16; iPtr++)
+					if (config.logicalAddresses[iPtr])
+						LogicalAddresses->Set((CecLogicalAddress)iPtr);
+			}
 
-      if (ServerVersion >= CecServerVersion::Version1_6_0)
-      {
-        FirmwareVersion          = config.iFirmwareVersion;
-        PowerOffDevicesOnStandby = config.bPowerOffDevicesOnStandby == 1;
-      }
+			if (ServerVersion >= CecServerVersion::Version1_6_0)
+			{
+				FirmwareVersion          = config.iFirmwareVersion;
+				PowerOffDevicesOnStandby = config.bPowerOffDevicesOnStandby == 1;
+				ShutdownOnStandby        = config.bShutdownOnStandby == 1;
+			}
+
+			if (ServerVersion >= CecServerVersion::Version1_6_2)
+				DeviceLanguage = gcnew System::String(config.strDeviceLanguage);
 		}
 
 		property System::String ^     DeviceName;
@@ -660,11 +669,12 @@ namespace CecSharp
 		property CecLogicalAddresses ^PowerOffDevices;
 		property bool                 PowerOffScreensaver;
 		property bool                 PowerOffOnStandby;
-    property bool                 SendInactiveSource;
-    property CecLogicalAddresses ^LogicalAddresses;
-    property uint16_t             FirmwareVersion;
-    property bool                 PowerOffDevicesOnStandby;
-
+		property bool                 SendInactiveSource;
+		property CecLogicalAddresses ^LogicalAddresses;
+		property uint16_t             FirmwareVersion;
+		property bool                 PowerOffDevicesOnStandby;
+		property bool                 ShutdownOnStandby;
+		property System::String ^     DeviceLanguage;
 		property CecCallbackMethods ^ Callbacks;
 	};
 
@@ -676,12 +686,14 @@ namespace CecSharp
 	typedef int (__stdcall *COMMANDCB)(const CEC::cec_command &command);
 	typedef int (__stdcall *CONFIGCB) (const CEC::libcec_configuration &config);
 	typedef int (__stdcall *ALERTCB)  (const CEC::libcec_alert, const CEC::libcec_parameter &data);
+	typedef int (__stdcall *MENUCB)   (const CEC::cec_menu_state newVal);
 
 	static LOGCB              g_logCB;
 	static KEYCB              g_keyCB;
 	static COMMANDCB          g_commandCB;
 	static CONFIGCB           g_configCB;
 	static ALERTCB            g_alertCB;
+	static MENUCB             g_menuCB;
 	static CEC::ICECCallbacks g_cecCallbacks;
 
 	int CecLogMessageCB(void *cbParam, const CEC::cec_log_message &message)
@@ -719,6 +731,13 @@ namespace CecSharp
 		return 0;
 	}
 
+	int CecMenuCB(void *cbParam, const CEC::cec_menu_state newVal)
+	{
+		if (g_menuCB)
+			return g_menuCB(newVal);
+		return 0;
+	}
+
 	#pragma managed
 	// delegates for the unmanaged callback methods
 	public delegate int CecLogMessageManagedDelegate(const CEC::cec_log_message &);
@@ -726,6 +745,7 @@ namespace CecSharp
 	public delegate int CecCommandManagedDelegate(const CEC::cec_command &);
 	public delegate int CecConfigManagedDelegate(const CEC::libcec_configuration &);
 	public delegate int CecAlertManagedDelegate(const CEC::libcec_alert, const CEC::libcec_parameter &);
+	public delegate int CecMenuManagedDelegate(const CEC::cec_menu_state newVal);
 
 	// callback method interface
 	public ref class CecCallbackMethods
@@ -792,6 +812,11 @@ namespace CecSharp
 			return 0;
 		}
 
+		virtual int ReceiveMenuStateChange(CecMenuState newVal)
+		{
+			return 0;
+		}
+
 	protected:
 		// managed callback methods
 		int CecLogMessageManaged(const CEC::cec_log_message &message)
@@ -851,6 +876,16 @@ namespace CecSharp
 			return iReturn;
 		}
 
+		int CecMenuManaged(const CEC::cec_menu_state newVal)
+		{
+			int iReturn(0);
+			if (m_bHasCallbacks)
+			{
+				iReturn = m_callbacks->ReceiveMenuStateChange((CecMenuState)newVal);
+			}
+			return iReturn;
+		}
+
 		void DestroyDelegates()
 		{
       m_bHasCallbacks = false;
@@ -860,6 +895,8 @@ namespace CecSharp
 				m_logMessageGCHandle.Free();
 				m_keypressGCHandle.Free();
 				m_commandGCHandle.Free();
+				m_alertGCHandle.Free();
+				m_menuGCHandle.Free();
 			}
 		}
 
@@ -901,6 +938,12 @@ namespace CecSharp
         g_alertCB                  = static_cast<ALERTCB>(System::Runtime::InteropServices::Marshal::GetFunctionPointerForDelegate(m_alertDelegate).ToPointer());
         g_cecCallbacks.CBCecAlert  = CecAlertCB;
 
+        // create the delegate method for the menu callback
+        m_menuDelegate             = gcnew CecMenuManagedDelegate(this, &CecCallbackMethods::CecMenuManaged);
+        m_menuGCHandle             = System::Runtime::InteropServices::GCHandle::Alloc(m_menuDelegate);
+        g_menuCB                   = static_cast<MENUCB>(System::Runtime::InteropServices::Marshal::GetFunctionPointerForDelegate(m_menuDelegate).ToPointer());
+        g_cecCallbacks.CBCecMenuStateChanged = CecMenuCB;
+
         delete context;
         m_bDelegatesCreated = true;
       }
@@ -926,6 +969,10 @@ namespace CecSharp
 		static System::Runtime::InteropServices::GCHandle m_alertGCHandle;
 		CONFIGCB                                          m_alertCallback;
 
+		CecMenuManagedDelegate ^                          m_menuDelegate;
+		static System::Runtime::InteropServices::GCHandle m_menuGCHandle;
+		MENUCB                                            m_menuCallback;
+
 		CecCallbackMethods ^ m_callbacks;
 	  bool                 m_bHasCallbacks;
     bool                 m_bDelegatesCreated;
