@@ -38,16 +38,16 @@ namespace CEC
   class CCECPlaybackDevice : public CCECBusDevice
   {
   public:
-    CCECPlaybackDevice(CCECProcessor *processor, cec_logical_address address, uint16_t iPhysicalAddress = 0);
+    CCECPlaybackDevice(CCECProcessor *processor, cec_logical_address address, uint16_t iPhysicalAddress = CEC_INVALID_PHYSICAL_ADDRESS);
     virtual ~CCECPlaybackDevice(void) {};
 
-    virtual cec_deck_info GetDeckStatus(void);
-    virtual cec_deck_control_mode GetDeckControlMode(void);
+    cec_deck_info GetDeckStatus(void);
+    cec_deck_control_mode GetDeckControlMode(void);
 
-    virtual void SetDeckStatus(cec_deck_info deckStatus);
-    virtual void SetDeckControlMode(cec_deck_control_mode mode);
+    void SetDeckStatus(cec_deck_info deckStatus);
+    void SetDeckControlMode(cec_deck_control_mode mode);
 
-    virtual bool TransmitDeckStatus(cec_logical_address dest);
+    bool TransmitDeckStatus(cec_logical_address dest);
 
   protected:
     cec_deck_info         m_deckStatus;
